
#ifndef TEL_TOOLS_H
#define TEL_TOOLS_H


#ifdef __cplusplus
extern "C" {
#endif

/*
#define alloc_line(m) memb_alloc(m)
#define dealloc_line(m,line)  if(LOCATED_IN_RAM(line)) memb_free(m,line)
*/


#ifndef alloc_line
char *alloc_line(void);
#endif

#ifndef dealloc_line
void dealloc_line(char *line);
#endif


#define CONF_NUMLINES		 16
#define CONF_LINELEN_OUT	256


#define ftp_alloc_line() alloc_line()
#define ftp_dealloc_line(line) dealloc_line(line)

#define tel_alloc_line() alloc_line()
#define tel_dealloc_line(line) dealloc_line(line)


typedef struct
{
  unsigned char InteligentTerm;
  unsigned char state;
  unsigned char ActiveCommand;
} term_state;


#ifdef CONF_NUMLINES

 #define sendline(_lines,_line,_TmpNumsent,_NUM_LINES) sendlineL(_lines,_line,_TmpNumsent)
 int sendlineL(char *lines[], char *line, unsigned char TmpNumsent);
 #define acked(_lines,_numsent,_NUM_LINES)	 ackedL(_lines,_numsent)
 void ackedL(char *lines[], unsigned char *numsent);

 void sendopt(char *lines[], unsigned char option, unsigned char value, unsigned char TmpNumsent);
 void close_lines(char *lines[]);
 int senddata(char *lines[], unsigned char *numsent);
 void IAC(char *lines[], unsigned char c, term_state *TS, unsigned char TmpNumsent);
 void shell_output(char *lines[], const char *str1, const char *str2, unsigned char TmpNumsent);
 int shell_output1(char *lines[], const char *str, unsigned char TmpNumsent);

#else

 int sendline(char *lines[], char *line, unsigned char TmpNumsent, int NUM_LINES);
 void acked(char *lines[], unsigned char *numsent, int NUM_LINES);
 void sendopt(char *lines[], unsigned char option, unsigned char value, unsigned char TmpNumsent, int NUM_LINES);
 void close_lines(char *lines[], int NUM_LINES);
 int senddata(char *lines[], unsigned char *numsent, int NUM_LINES);
 void IAC(char *lines[], unsigned char c, term_state *TS, unsigned char TmpNumsent, int NUM_LINES);
 void shell_output(char *lines[], const char *str1, const char *str2, unsigned char TmpNumsent, int NUM_LINES);
 int shell_output1(char *lines[], const char *str, unsigned char TmpNumsent, int NUM_LINES);

#endif


int store_char(char *buf, unsigned char *bufptr, unsigned char c, unsigned bufsize);


#define ISO_nl       0x0a
#define ISO_cr       0x0d


#define STATE_NORMAL 0
#define STATE_IAC    1
#define STATE_WILL   2
#define STATE_WONT   3
#define STATE_DO     4
#define STATE_DONT   5
#define STATE_CLOSE  6
#define STATE_AFTER_IAC 7


#define IAC_IAC   255
#define IAC_WILL  251
#define IAC_WONT  252
#define IAC_DO    253
#define IAC_DONT  254

typedef enum
{
  IAC_BIN_XMIT    = 0,	/**< Allows transmission of binary data. */
  IAC_ECHO_DATA   = 1,	/**< Causes server to echo back all keystrokes. */
  IAC_RECONNECT   = 2,	/**< Reconnects to another TELNET host. */
  IAC_SUPPRESS_GA = 3,	/**< Disables Go Ahead! command. */
  IAC_MESSAGE_SZ  = 4,	/**< Conveys approximate message size. */
  IAC_OPT_STATUS  = 5,	/**< Lists status of options. */
  IAC_TIMING_MARK = 6,  /**< Marks a data stream position for reference. */
  IAC_RC_XMT_ECHO = 7,	/**< Allows remote control of terminal printers. */
  IAC_LINE_WIDTH  = 8,  /**< Sets output line width. */
  IAC_PAGE_LENGTH = 9,  /**< Sets page length in lines. */
  IAC_CR_USE      = 10,	/**< Determines handling of carriage returns. */
  IAC_HORIZ_TABS  = 11, /**< Sets horizontal tabs. */
  IAC_HR_TAB_USE  = 12, /**< Determines handling of horizontal tabs. */
  IAC_FF_USE      = 13, /**< Determines handling of form feeds. */
  IAC_VR_TABS     = 14, /**< Sets vertical tabs. */
  IAC_VR_TAB_USE  = 15, /**< Determines handling of vertical tabs. */
  IAC_LF_USE      = 16, /**< Determines handling of line feeds. */
  IAC_EXT_ASCII   = 17, /**< Defines extended ASCII characters. */
  IAC_LOGOUT      = 18, /**< Allows for forced log-off. */
  IAC_BYTE_MACRO  = 19, /**< Defines byte macros. */
  IAC_DATA_TERM   = 20, /**< Allows subcommands for Data Entry to be sent. */
  IAC_SUPDUP      = 21, /**< Allows use of SUPDUP display protocol. */
  IAC_SUPDUP_OUTP = 22, /**< Allows sending of SUPDUP output. */
  IAC_SEND_LOCATE = 23, /**< Allows terminal location to be sent. */
  IAC_TERM_TYPE   = 24, /**< Allows exchange of terminal type information. */
  IAC_END_RECORD  = 25, /**< Allows use of the End of record code (0xEF). */
  IAC_TACACS_ID   = 26, /**< User ID exchange used to avoid more than 1 log-in. */
  IAC_OUTPUT_MARK = 27, /**< Allows banner markings to be sent on output. */
  IAC_TERM_LOC_ID = 28,	/**< A numeric ID used to identify terminals. */
  IAC_3270_REGIME = 29,	/**< Allows emulation of 3270 family terminals. */
  IAC_X3_PAD      = 30, /**< Allows use of X.3 protocol emulation. */
  IAC_WIN_SIZE    = 31, /**< Conveys window size for emulation screen. */
  IAC_TERM_SPEED  = 32,	/**< Conveys baud rate information. */
  IAC_REMOTE_FLOW = 33, /**< Provides flow control (XON, XOFF). */
  IAC_LINEMODE    = 34, /**< Provides linemode bulk character transactions. */

  IAC_EXTENDED    = 255 /**< Options list  Extended options list. */
} IAC_CONSTANTS;


#ifdef __cplusplus
  }
#endif


#endif /* TEL_TOOLS_H */
