/*
 * Copyright (c) 2003, Adam Dunkels.
 * Copyright (c) 2010-2016 J. Fojtik
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack
 *
 * $Id: telnetd.c,v 1.2 2006/06/07 09:43:54 adam Exp $
 * Modified by Jaroslav Fojtik 2010 - 2016
 */
#include "uip.h"
#include "../misc/memb.h"
#include "telnetd.h"

#include "shell.h"

#include <string.h>

#define TELNET_PORT  23




/*---------------------------------------------------------------------------*/
void shell_quit(telnetd_state *s, char *str)
{
  s->TS.state = STATE_CLOSE;
}


/*---------------------------------------------------------------------------*/
void shell_prompt(telnetd_state *s, const char *str)
{
  char *line;
  line = tel_alloc_line();
  if(line != NULL) {
    strncpy(line, str, TELNETD_CONF_LINELEN_OUT);
    sendline(s->lines, line, s->numsent, TELNETD_CONF_NUMLINES);
  }
}


/*---------------------------------------------------------------------------*/
/** Process a new data blob received. */
static void newdata(telnetd_state *s)
{
  u16_t len;
  char *dataptr = (char *)uip_appdata;  
  
  len = uip_datalen();
  
  while(len>0 && s->bufptr<sizeof(s->buf))
  {       
    --len;

    IAC(s->lines,*dataptr,&s->TS, s->numsent);

    if(s->TS.state==STATE_NORMAL)    
      if(store_char(s->buf,&s->bufptr,*dataptr,sizeof(s->buf)))
        {shell_input(s, s->buf); s->bufptr = 0;}    

    ++dataptr;
  }  
}


/*---------------------------------------------------------------------------*/ 
/** Hook to uIP stack. */
static void telnetd_appcall(void)
{
telnetd_state *s = (telnetd_state *)&(uip_conn->appstate);

  //if(uip_conn->lport!=HTONS(TELNET_PORT)) return;

  if(uip_connected())
  {
    /*    tcp_markconn(uip_conn, &s);*/
    memset(s->lines, 0, sizeof(s->lines));    
    s->bufptr = 0;
    s->TS.state = STATE_NORMAL;
    s->TS.InteligentTerm = 0;
    s->TS.ActiveCommand = 0xFF;		// No Cmd

    shell_start(s);
  }

  if(s->TS.state == STATE_CLOSE) 
  {
    s->TS.state = STATE_NORMAL;
    uip_close();
    return;
  }
  
  if(uip_closed() || uip_aborted() || uip_timedout())
  {
    close_lines(s->lines);
    return;				// The connection is no longer valid.
  }
  
  if(uip_acked()) 
  {    
    acked(s->lines,&s->numsent,TELNETD_CONF_NUMLINES);
  }
  
  if(uip_newdata()) 
  {
    newdata(s);
  }
  
  if(uip_rexmit() || uip_newdata() || uip_connected() || uip_poll()) 
  {
    senddata(s->lines,&s->numsent);
  }
}


/*---------------------------------------------------------------------------*/
/** External TELNET initialisation point. */
void telnetd_init(void)
{
  uip_listen(HTONS(TELNET_PORT),telnetd_appcall);
  memb_init(&linemem);
  shell_init();
}

/*---------------------------------------------------------------------------*/
