#ifndef __EMAC_H
#define __EMAC_H

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
  EMAC_ERR_JUMBO 	=      1,	/**< Very big packet has been received or receiving FIFO is not synchronised. */
  EMAC_ERR_TOOSMALL	=      2,	/**< Illegal packet has been received or Receiving FIFO is not synchronised. */
  EMAC_ERR_OVERLOAD     =      4,	/**< Cannot allocate a new buffer for packet, possible overload or eth (DTP) thread hanged. */
  EMAC_ERR_DMA		=      8,	/**< DMA controller is posibly stucked. */
  EMAC_ERR_ETHER	=   0x10,	/**< Tx transmitting FIFO stucked. */
  EMAC_ERR_BAD_IRQ	=   0x20,	/**< IRQ is called recursivelly. */
  EMAC_ERR_ST_PKT	=   0x40,	/**< Something is in a transmit buffer, but Tx buffer is expected to be empty. */
  EMAC_ERR_RX_OVERRUN	=   0x80,	/**< Rx FIFO overrun. */
  EMAC_ERR_RX_ISR_NACC	=  0x100,	/**< Rx is not servicable inside IRQ, must be serviced in separate job. */
  EMAC_ERR_TX_OVERFLOW	=  0x200,	/**< Tx packet cache overloaded. */
  EMAC_ERR_RX_OVERFLOW  =  0x400,	/**< Rx packet cache overloaded. */
  EMAC_ERR_TX_ERROR	=  0x800,	/**< Tx error reported by device. */
  EMAC_ERR_TX_RETRANSMIT= 0x1000,	/**< Tx did not transmit a packet correctly. */

  EMAC_ERR_RESTARTED	= 0x8000,	/**< Somethink went very bad and all peripheral must be restarted. */
} EMAC_ERRORS;

extern unsigned short EMAC_ErrorFlags;


	/* MAC address definition.  The MAC address must be unique on the network. */
#define emacETHADDR0 0
#define emacETHADDR1 0xbd
#define emacETHADDR2 0x33
#define emacETHADDR3 0x02
#define emacETHADDR4 0x64
#define emacETHADDR5 0x24


	/* EMAC Memory Buffer configuration for 16K Ethernet RAM. */
#define NUM_RX_FRAG         4           /* Num.of RX Fragments 4*1536= 6.0kB */
#define NUM_TX_FRAG         2           /* Num.of TX Fragments 2*1536= 3.0kB */
#define ETH_FRAG_SIZE       1536        /* Packet Fragment size 1536 Bytes   */

#define ETH_MAX_FLEN        1536        /* Max. Ethernet Frame Size          */

	/* Prototypes. */
int EthInitModule(void);

portBASE_TYPE  vInitEMAC(void);
unsigned short ReadFrameBE_EMAC(void);
void           vIncrementTxLength(unsigned long ulLength);
void           CopyFromFrame_EMAC(void *Dest, unsigned short Size);
void           DummyReadFrame_EMAC(unsigned short Size);
unsigned short StartReadFrame(void);
void           EndReadFrame(void);
unsigned int   CheckFrameReceived(void);
void           vInitialiseSend(void);
unsigned int   Rdy4Tx(void);
void		vSendBufferToMAC(unsigned char **ppus, portSHORT ulNextTxSpace);
void vEMACWaitForInput( void );
unsigned int uiGetEMACRxData(unsigned char **ucBuffer, int Timeout);


	/* Packet cache */
unsigned char *GetFreePkt(char CallFromIRQ);
void DisposePkt(unsigned char *pPacket, char CallFromIRQ);
void InitPktPool(void);
unsigned GetPktNum(unsigned char *pPacket);


#ifdef __cplusplus
 }
#endif

#endif

/*----------------------------------------------------------------------------
 * end of file
 *---------------------------------------------------------------------------*/
